#include <msp430x16x.h>
#include "lcd.h"


#define LCD_RESET_LOW     P3OUT &= ~BIT6
#define LCD_RESET_HIGH    P3OUT |= BIT6

#define CS_LOW    P5OUT &= ~0x01;
#define CS_HIGH   P5OUT |= 0x01;

void DelayS (unsigned long a) { while (--a!=0); }

unsigned int i,j;

void InitLCD(void) {

  // USART1 initialization for SPI communications //

  // Enable USART0 SPI mode
  U1ME |= USPIE1;

  /* USART Control Register */
  // 8-bit, SPI Master
  UCTL1 = CHAR+SYNC+MM;	

  /* USART Transmit Control Register */
  // Select SMCLK and 3-pin SPI select, Clock Polarity
  //U1TCTL = CKPL+SSEL0+SSEL1+STC+CKPH;	
  //U1TCTL = SSEL0+SSEL1+STC;
  U1TCTL = SSEL0+SSEL1+STC+CKPL;

  /* USART Baud Rate Control Register 0 and 1 */
  /* USART Modulation Control Register */
  // Set BoudRate - no devider
  UBR01   = 0x04;
  UBR11   = 0x00;
  UMCTL1  = 0x00;

  // Setup P3 for SPI1 operation
  P5SEL |= 0x0A;		// P3.1 and P3.3 SPI option select

  P5SEL &= ~0x01;		// P3.0 as GPIO
  P5OUT |= 0x01;		// CS high
  P5DIR |= 0x01;		// P3.0 output direction

  // Reset Pin
  P3SEL &= ~0x40;		// P3.6 as GPIO
  P3OUT |= 0x40;		// reset high
  P3DIR |= 0x40;		// P3.6 output direction

  // Backlight
  P3SEL &= ~BIT7;               // P1.2 as GPIO
  P3OUT &= ~BIT7;                // Turn on light
  P3DIR |= BIT7;                // P1.2 output direction
}


void WriteSpiCommand(unsigned int data){

  // Software clock for first bit (must be high)
  P5SEL &= ~0x0A; // P3.1 and P3.3 as GPIO
  P5OUT |= 0x08;  // P3.3  -> HIGH
  P5OUT &= ~0x02; // P3.1  -> LOW
  P5DIR |= 0x0A;  // P3.1 and P3.3 -> OUTPUT

    // CS Low
  CS_LOW;

  P5OUT &= ~0x08; // P4.5 -> LOW
  P5OUT |= 0x08;  // P4.5 -> HIGH


  P5SEL |= 0x0A;  // P4.3 and P4.5 SPI option select

  // Write byte
  TXBUF1 = data & 0xFF;

  // wait to send data
  while ((U1TCTL & TXEPT) == 0);	

  // CS High
  CS_HIGH;
}

void WriteSpiData(unsigned int data){

  // Software clock for first bit (must be high)
  P5SEL &= ~0x0A; // P4.3 and P4.5 as GPIO
  P5OUT |= 0x0A;  // P4.3 and P4.5  -> HIGH
  P5DIR |= 0x0A;  // P4.3 and P4.5 -> OUTPUT

  // CS Low
  CS_LOW;

  P5OUT &= ~0x08; // P4.5 -> LOW
  P5OUT |= 0x08;  // P4.5 -> HIGH


  P5SEL |= 0x0A;  // P4.3 and P4.5 SPI option select

  // Write byte
  TXBUF1 = data & 0xFF;

  // wait to send data
  while ((U1TCTL & TXEPT) == 0);	

  // CS High
   CS_HIGH;
}


void Backlight(unsigned char state) {

  if(state == BKLGHT_LCD_ON)
      P3OUT |= BIT7;    //Turn on light
  else
      P3OUT &= ~BIT7;    //Turn off light
}

void SetContrast(unsigned char contrast) {

#ifdef GE12
  WriteSpiCommand(CONTRAST);
  WriteSpiData(0x20+contrast);    // contrast
#else

#endif

}

void LCDSettings(void) {

#ifdef GE12
 // Hardware reset 
 LCD_RESET_LOW; 
 DelayS(2000); 
 LCD_RESET_HIGH; 
 DelayS(2000); 
 
 // Sleep out (command 0x11) 
 WriteSpiCommand(SLEEPOUT); 
 
 // Inversion off (command 0x20) 
 WriteSpiCommand(INVERSIONOFF); // seems to be required for this controller 
 
 // Color Interface Pixel Format (command 0x3A) 
 WriteSpiCommand(PIXELFORMAT); 
 WriteSpiData(0x03); // 0x03 = 12 bits-per-pixel 
 
 WriteSpiCommand(ACCESSCTRL); 
 WriteSpiData(0x08);
#else

  // Display vontrol
  WriteSpiCommand(DISCTL);

  WriteSpiData(0x00); // default
  WriteSpiData(0x20); // (32 + 1) * 4 = 132 lines (of which 130 are visible)
  WriteSpiData(0x0a); // default


  // COM scan
  WriteSpiCommand(COMSCN);
  WriteSpiData(0x01);  // Scan 1-80

  // Internal oscilator ON
  WriteSpiCommand(OSCON);

  // Sleep out
  WriteSpiCommand(SLPOUT);

  // Voltage control
  WriteSpiCommand(VOLCTR);
  // WriteSpiData(0x1F); // middle value of V1
  WriteSpiData(0x25); // middle value of V1
  WriteSpiData(0x03); // middle value of resistance value

  // Temperature gradient
  WriteSpiCommand(TMPGRD);
  WriteSpiData(0x00); // default

  // Power control
  WriteSpiCommand(PWRCTR);
  WriteSpiData(0x0f);   // referance voltage regulator on, circuit voltage follower on, BOOST ON

  // Normal display
  WriteSpiCommand(DISNOR);

  // Inverse display
  WriteSpiCommand(DISINV);

  // Partial area off
  WriteSpiCommand(PTLOUT);

  // Data control
  WriteSpiCommand(DATCTL);
  WriteSpiData(0x00); // all inversions off, column direction
  // WriteSpiData(0x00); // BGR sequence
  WriteSpiData(0x01); // RGB sequence
  WriteSpiData(0x02); // 4k color

  // Page Address set
  WriteSpiCommand(PASET);
  WriteSpiData(0);
  WriteSpiData(131);

  // Page Column set
  WriteSpiCommand(CASET);
  WriteSpiData(0);
  WriteSpiData(131);

  // wait ~100ms
  DelayS(10000);

#endif

}


void LCDWrite130x130bmp(void) {

  int size;
  size = 130*130;

#ifdef GE12

  // Display OFF
  WriteSpiCommand(DISPLAYOFF);

  // WRITE MEMORY
  WriteSpiCommand(MEMWRITE);

  // write to memory
  for(j=0; j<size; j++) {
    WriteSpiData(0xFF);
    //WriteSpiData(bmp[i]);
  }

  // Display On
  WriteSpiCommand(DISPLAYON);	

#else

  // WRITE MEMORY
  WriteSpiCommand(RAMWR);

  for(j=0; j<size; j++) {
    //WriteSpiData(bmp[j]);
    WriteSpiData(0xFF);
  }

  // wait ~100ms
  DelayS(10000);

  // Display On
  WriteSpiCommand(DISON);	

#endif


}

void ClearLCD(void) {

  int size;
  size = 130*130;

#ifdef GE12

  // Page address set
  WriteSpiCommand(PAGEADDRSET);
  WriteSpiData(0);
  WriteSpiData(131);

  // Column address set
  WriteSpiCommand(COLADDRSET);
  WriteSpiData(0);
  WriteSpiData(131);

  // WRITE MEMORY
  WriteSpiCommand(MEMWRITE);

  // write to memory
  for(j=0; j<size; j++) {
    WriteSpiData(0xFF);
  }

  // Display On
  WriteSpiCommand(DISPLAYON);	

#else

  // Page address set
  WriteSpiCommand(PASET);
  WriteSpiData(0);
  WriteSpiData(131);

  // Page Column set
  WriteSpiCommand(CASET);
  WriteSpiData(0);
  WriteSpiData(131);

  // WRITE MEMORY
  WriteSpiCommand(RAMWR);

  // write to memory
  for(j=0; j<size; j++) {
    WriteSpiData(0xFF);
  }

  // wait ~100ms
  DelayS(10000);

  // Display On
  WriteSpiCommand(DISON);	

#endif
}
